@@ -1,13 +1,14 @@
 #include "squid.h"
-
-#include "ConfigParser.h"
-#include "adaptation/Config.h"
 #include "adaptation/AccessRule.h"
+#include "adaptation/Config.h"
 #include "adaptation/DynamicGroupCfg.h"
 #include "adaptation/Service.h"
 #include "adaptation/ServiceFilter.h"
 #include "adaptation/ServiceGroups.h"
-
+#include "ConfigParser.h"
+#include "Debug.h"
+#include "StrList.h"
+#include "wordlist.h"
 
 Adaptation::ServiceGroup::ServiceGroup(const String &aKind, bool allSame):
         kind(aKind), method(methodNone), point(pointNone),
@@ -22,7 +23,7 @@ Adaptation::ServiceGroup::~ServiceGroup()
 void
 Adaptation::ServiceGroup::parse()
 {
-    ConfigParser::ParseString(&id);
+    id = ConfigParser::NextToken();
 
     wordlist *names = NULL;
     ConfigParser::ParseWordList(&names);
@@ -75,7 +76,7 @@ Adaptation::ServiceGroup::finalize()
                     baselineKey = service->cfg().key;
                     baselineBypass = service->cfg().bypass;
                 } else if (baselineBypass != service->cfg().bypass) {
-                    debugs(93,0, "WARNING: Inconsistent bypass in " << kind <<
+                    debugs(93, DBG_CRITICAL, "WARNING: Inconsistent bypass in " << kind <<
                            ' ' << id << " may produce surprising results: " <<
                            baselineKey << " vs. " << serviceId);
                 }
@@ -193,14 +194,12 @@ Adaptation::ServiceGroup::findLink(const ServiceFilter &filter, Pos &pos) const
     return !allServicesSame && findService(filter, pos);
 }
 
-
 /* ServiceSet */
 
 Adaptation::ServiceSet::ServiceSet(): ServiceGroup("adaptation set", true)
 {
 }
 
-
 /* SingleService */
 
 Adaptation::SingleService::SingleService(const String &aServiceId):
@@ -210,14 +209,12 @@ Adaptation::SingleService::SingleService(const String &aServiceId):
     services.push_back(aServiceId);
 }
 
-
 /* ServiceChain */
 
 Adaptation::ServiceChain::ServiceChain(): ServiceGroup("adaptation chain", false)
 {
 }
 
-
 /* DynamicServiceChain */
 
 Adaptation::DynamicServiceChain::DynamicServiceChain(
@@ -313,7 +310,6 @@ Adaptation::ServicePlan::print(std::ostream &os) const
            (atEof ? ".]" : "]");
 }
 
-
 /* globals */
 
 Adaptation::Groups &
